
/**************************************************************************************
* This file is part of the FrameworkTest project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2001-2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

//////////////////////////
//WinMain - This is the win32 entry point to our project
//////////////////////////
#define WIN32_MEAN_AND_LEAN
#include <windows.h>
#include <tchar.h>
#include <stdio.h>

//include our GameFramework header file
#include "GameFramework.h"

//some "nasty" globals...boo! We hate you global!
//
HWND g_hWnd = NULL;
HINSTANCE g_hInstance = NULL;
TCHAR g_szAppClass[] = TEXT("FrameWorktest");
TCHAR g_szAppTitle[] = TEXT("FrameWorktest v1.3");
BOOL g_bIsActive = FALSE;

//a pointer to our rendererFactory object
rendererFactory  *pGraphicsFactory = NULL;

//version 1.3
//Added the following #define to reference our texture image
#define LOGO 1

//function declarations
LRESULT WINAPI msgProc( HWND, UINT, WPARAM, LPARAM);
HRESULT createProgram();
void updateProgram(FLOAT);
void renderProgram();
void destroyProgram();

//include our library
#pragma comment(lib, "GameFramework.lib")

// Name: WinMain
// Description: Win32 entry point.
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASSEX	wndclass;
	HWND		hWnd;
	MSG			msg;
	HRESULT		hr;
	
	// Set up window attributes
	wndclass.cbSize			= sizeof(wndclass);
	wndclass.style			= CS_HREDRAW | CS_VREDRAW |
							  CS_OWNDC   | CS_DBLCLKS;
	wndclass.lpfnWndProc	= msgProc;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInstance;
	wndclass.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wndclass.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground	= (HBRUSH)(COLOR_WINDOW);
	wndclass.lpszMenuName	= NULL;
	wndclass.lpszClassName	= g_szAppClass;	// Registered Class Name
	wndclass.hIconSm		= LoadIcon( NULL, IDI_APPLICATION );
		
	//register our new window class with the Windows OS
	if( RegisterClassEx( &wndclass ) == 0 ) {
		// Do error logic here
		exit(1);
	}
	
	// Create the main window
	//Version1.1
	//Modified the window parameters to just allow us to display the
	//main window as a 640x480 sized windowed app rather than a fullscreen one
	hWnd = CreateWindow(	
		g_szAppClass,				// Class Name
		g_szAppTitle,			// Name Displayed on Title Bar
		WS_OVERLAPPEDWINDOW,           // Window Style...for now POPUP is fine..
		0,                  // x position (top left)
		0,                  // y position (top left)
		640, //x position (bottom right)
		480, //y position (bottom right)
		NULL,
		NULL,
		hInstance,
		NULL );

	
	ShowWindow(hWnd, nCmdShow);       //we must first tell Windows to show our app
	UpdateWindow(hWnd);	              //updates the client by sending an initial WM_PAINT msg

	g_hWnd = hWnd;                    //save our main window handle
	g_hInstance = hInstance;          //save our main application instance


	
	//create our application
	hr = createProgram();
	if(FAILED(hr))
	{
		//we got a major error, so get outta here!
		destroyProgram();
		return(0);
	}

	//main message loop
	//keep looping through our game until the user quits it 
	//(thereby sending a WM_QUIT message to the application)
	ZeroMemory( &msg, sizeof(msg) );
    while( msg.message != WM_QUIT )
	{
        if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
		{
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
        else 
		{
			//if our application is NOT minimized on the taskbar,
			//then let's go through our update and rendering
			//loops
			if(g_bIsActive)
			{

				// Update application
				updateProgram(1.0f);

				// render the current scene
				renderProgram();
			}
		}
    }

	// Cleanup the environment
	destroyProgram();
	UnregisterClass(g_szAppClass, hInstance);

	//return back to windows
	return( (int) msg.wParam);
};

//Name: msgProc
//Description: This is our main window message handler
LRESULT WINAPI msgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
    {
		case WM_ACTIVATE:
			//WM_ACTIVATE is sent whenever the application has the focus..
			//so if we minimize our app, we LOSE the focus, and therefore
			//shouldn't bother wasting clock cycles/graphics hardware
			//to render what we can't see!
			g_bIsActive = (BOOL)wParam;
			break;

		case WM_KEYDOWN:
			// Handle any non-accelerated key commands
            switch (wParam)
            {
			case VK_ESCAPE:
				//if the user hits ESC, then we'll just
				//quit from this application
				PostMessage(g_hWnd, WM_QUIT, 0, 0);
				return 0;
				
			};
			break;

        case WM_DESTROY:
			//WM_DESTROY is what's sent by the Windows messaging system
			//when our message queue receives the WM_QUIT message from
			//our application...just cleans up everything and quits 
			//the program..
            PostQuitMessage( 0 );
			return 0;
		
    }

	//whatever message was sent here, wasn't handled by us, so return the 
	//message back to the queue for the next windows application (ie. a 
	//background task or something)..
    return DefWindowProc( hWnd, msg, wParam, lParam );
}

//////////////////////////////////////////////////////////
// createProgram
// This function is responsible for initializing our
// game subsystems...ie. sound, input, graphics, etc..
//////////////////////////////////////////////////////////
HRESULT createProgram(){
	
	HRESULT hr = S_OK;
	
	//first let's create an instance of our rendererFactory
	pGraphicsFactory = new rendererFactory(g_hInstance);
	
	//we'll now tell it to read our settings from the
	//accompanying .ini file
	hr = pGraphicsFactory->initInterface("data\\config\\FrameworkTest.ini", g_hWnd);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	
	//initialize our rendererInterface object
	rendererInterface *pInterface = NULL;

	//grab the interface that was created by the 
	//rendererFactory
	pInterface = pGraphicsFactory->getInterface();
	if(pInterface == NULL)
	{
		return E_FAIL;
	}

	//now let's create the main graphics device!
	//due to proper class inheritance, we're able to call
	//the methods from our rendererInterface and have them
	//resolved automatically to our desired graphics API,
	//making these calls API independant!

	//Version 1.1
	//Modified the device code to just render to a 640x480 window
	hr = pInterface->createDevice(g_hWnd,
								  640,
								  480,
								  16,
								  TRUE);
	if(FAILED(hr))
	{
		return E_FAIL;
	}

	//set our clearing color to blue
	pInterface->setClearColor(0.0f, 0.0f, 1.0f, 1.0f);


	//v1.3
	//get the pointer to our texture interface to add our texture
	//
	hr = pInterface->getTextureInterface()->addTexture("data\\textures\\wazooPresents.bmp", LOGO);
	if(FAILED(hr)){
		return E_FAIL;
	}

	return S_OK;
}

////////////////////////////////////////////////////
// destroyProgram
// This function is responsible for handling all of the 
// cleanup of our subsystems..
////////////////////////////////////////////////////
void destroyProgram(){

	//if our rendererFactory exists, blow it away!
	if(pGraphicsFactory)
	{
		delete pGraphicsFactory;
		pGraphicsFactory = NULL;
	}


}

/////////////////////////////////////////////
// updateProgram
// This function is responsible for handling the updating
// to each object in our game, as well as processing input,
// etc..
/////////////////////////////////////////////
void updateProgram(FLOAT fTimeKey){

	


}

////////////////////////////////////////
// renderProgram
// This function is responsible for handling the rendering
// of our graphics...depending on what's happening in the
// game
///////////////////////////////////////
void renderProgram(){

	rendererInterface *pInterface = NULL;

	//again, get the pointer to our rendererInterface object
	pInterface = pGraphicsFactory->getInterface();
	if(pInterface == NULL)
		return;

	//let's begin our rendering loop
	if(SUCCEEDED(pInterface->beginRenderingScene()))
	{
		//TODO: render game stuff

		//v1.3 let our graphics pipeline know about our texture
		pInterface->getTextureInterface()->setTexture(LOGO);

		//V1.1
		//We want to render a simple square using our new VertexBuffer interface
		//V1.3
		//modified the parameters to support the new texture mapping vertices
		if(SUCCEEDED(pInterface->m_pVB->lockVB())){

			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(-1.0f, -1.0f, -10.0f),
				1.0f, 1.0f, 1.0f, 1.0f, D3DXVECTOR2(0.0f, 0.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(-1.0f, 1.0f, -10.0f),
				1.0f, 1.0f, 1.0f, 1.0f, D3DXVECTOR2(0.0f, 1.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(1.0f, -1.0f, -10.0f),
				1.0f, 1.0f, 1.0f, 1.0f, D3DXVECTOR2(1.0f, 0.0f));


			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(1.0f, -1.0f, -10.0f),
				1.0f, 1.0f, 1.0f, 1.0f, D3DXVECTOR2(1.0f, 0.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(-1.0f, 1.0f, -10.0f),
				1.0f, 1.0f, 1.0f, 1.0f, D3DXVECTOR2(0.0f, 1.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(1.0f, 1.0f, -10.0f),
				1.0f, 1.0f, 1.0f, 1.0f, D3DXVECTOR2(1.0f, 1.0f));


			pInterface->m_pVB->unlockVB();
		}

		//end our rendering loop, and signal the 
		//graphics hardware to blast the primitives
		//out to the device
		pInterface->endRenderingScene();
	}


}
