
#pragma warning(disable: 4786)

#include "OGLTextureManager.h"
#include <gl/glaux.h>

OGLTextureManager::OGLTextureManager(){

}

OGLTextureManager::~OGLTextureManager(){

	//1.3
	//added the following method to clean up our textures map container.
	//just use the stl::iterator to go through our map container and
	//remove the stored texture data
	for(std::map<int, GLuint>::iterator it = m_oTextures.begin(); 
		it != m_oTextures.end();
		it++){

		glDeleteTextures(1, &it->second );

	}

	//clear the map
	m_oTextures.clear();

}

HRESULT OGLTextureManager::addTexture(std::string strFilename, const int value){

	AUX_RGBImageRec *pBitmap = NULL;
		
	
	pBitmap = auxDIBImageLoad(strFilename.c_str());				
	
	if(!pBitmap){
		return E_FAIL;
	}

	GLuint tex;

	// Generate a texture with the associative texture ID stored in the array
	glGenTextures(1, &tex);
	
	// Bind the texture to the texture arrays index and init the texture
	glBindTexture(GL_TEXTURE_2D, tex);

	
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, pBitmap->sizeX, 
					  pBitmap->sizeY, GL_RGB, GL_UNSIGNED_BYTE, pBitmap->data);

	// Lastly, we need to tell OpenGL the quality of our texture map.  GL_LINEAR_MIPMAP_LINEAR
	// is the smoothest.  GL_LINEAR_MIPMAP_NEAREST is faster than GL_LINEAR_MIPMAP_LINEAR, 
	// but looks blochy and pixilated.  Good for slower computers though. 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);	

	
	m_oTextures.insert(std::make_pair(value, tex));

	if(pBitmap){
		delete pBitmap;
		pBitmap = NULL;
	}
	

	return S_OK;
}


void OGLTextureManager::setTexture(const int value){

	std::map<int, GLuint>::iterator it;
	it = m_oTextures.find(value);
	if (it == m_oTextures.end()) { 
		return;
	}
	else {
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, it->second);

	}


}

